const { Pool } = require('pg');
const GenericQuery = require('./classes/query.generic.class');
const Select = require('./classes/select.class');
const Insert = require('./classes/insert.class');
const Delete = require('./classes/delete.class');
const Update = require('./classes/update.class');
const Tape = require('./classes/tape.class');

/**
 * Class for communication with PostgreSQL database
 *
 * @class Database
 */
class Database {
  /**
   *
   * @param {Object} params
   * @param {String} params.host - Connection Host
   * @param {number} params.port - Connection Port
   * @param {String} params.database - DB Name
   * @param {String} params.user - DB User
   * @param {String} params.password - DB Password
   */
  constructor({ host, port, database, user, password }) {
    this._db = new Pool({ host, port, database, user, password });
  }

  /**
   * Returns `true` if a transaction is active; `false` otherwise
   *
   * @readonly
   */
  get transaction() {
    return this._transaction;
  }

  /**
   * Creates a GenericQuery object
   *
   * @param {String} query
   * @param {Array.<any>} data
   */
  query(query, data = []) {
    return new GenericQuery(this, query, data);
  }

  /**
   * Creates a Select object
   *
   * @param {String} table
   */
  select(table) {
    return new Select(this, table);
  }

  /**
   * Creates an Insert object
   *
   * @param {String} table
   */
  insert(table) {
    return new Insert(this, table);
  }

  /**
   * Creates a Delete object
   *
   * @param {String} table
   */
  delete(table) {
    return new Delete(this, table);
  }

  /**
   * Creates an Update object
   *
   * @param {String} table
   */
  update(table) {
    return new Update(this, table);
  }

  /**
   * Begins a transaction
   */
  begin() {
    return this.query('BEGIN');
  }

  /**
   * Commits an active transaction
   */
  commit() {
    return this.query('COMMIT');
  }

  /**
   * Creates and returns a new `Tape` instance
   */
  tape() {
    return new Tape(this);
  }
}

module.exports = Database;
